class Artist < Peta::Owner
  load_manifest
  belongs_to :user
  has_many :original_pictures
  has_many :pictures
  has_many :resource_pictures
  
  validates :name, :presence => true, :length => {:maximum => 30}
  validates :user_id, :numericality => true, :existence => {:both => false}
  validates :provider_id, :numericality => {:allow_nil => true}
  
  def supply_default
    self.name = 'no name' if self.name.blank?
    self.user_id = nil
  end
  
  def self.find_by_author author
    Artist.find( :first, :conditions => ['author_id = ?', author.id])
  end
  
  def self.index_list_where list
    'artists.provider_id is null'
  end
  
  def self.show_opt
    {:include => {:user => {:author => {}}} }
  end
  
  def self.export(dt = nil)
    opt = {}
    cond = if dt
      ['artists.artists.provider = 0 and artists.updated_at >= ?', dt]
    else
      'artists.artists.provider = 0'
    end
    opt.merge!({:conditions => cond}) 
    opt.merge!({:order => 'id'})
    Artist.find(:all, opt)
  end
  
end
