class GroundColor < Peta::Element
  load_manifest
  belongs_to :panel
  belongs_to :color
  
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :code, :presence => true, :numericality => {:greater_than_or_equal_to => 0, :less_than => 0x1000000}
  validates :orientation, :presence => true, :numericality => true, :inclusion => {:in => 0..1}
  validates :xy, :numericality => {:greater_than_or_equal_to => 0, :allow_blank => true}
  validates :wh, :numericality => {:greater_than_or_equal_to => 0, :allow_blank => true}
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  def self.by_author_list_includes
    {
      :panel => {
        :author => {}
      }
    }
  end
  
  def self.has_picture?
    false
  end
  
  def supply_default
    self.code ||= 0
    if self.panel
      self.t = self.panel.new_t 
      self.z = self.panel.new_z 
    end
  end
  
  def overwrite pid
    self.panel_id = pid
  end
  
  def div_offset
    xy ? xy : 0
  end
  
  def div_size
    wh ? wh : 100 - self.div_offset
  end
  
  def div_x
    if self.orientation == 0
      0
    else
      self.div_offset
    end
  end
  
  def div_y
    if self.orientation == 0
      self.div_offset
    else
      0
    end
  end
  
  def div_width
    if self.orientation == 0
      100
    else
      self.div_size
    end
  end
  
  def div_height 
    if self.orientation == 0
      self.div_size
    else
      100
    end
  end
  
  def style spot = nil, opacity = 20
    r = {
      'position' => 'absolute', 'z-index' => self.z, 
      'top' => self.div_y.to_s + '%', 'left' => self.div_x.to_s + '%', 
      'width' => self.div_width.to_s + '%', 'height' => self.div_height.to_s + '%', 
      'background-color' => '#' + format("%06x", self.code)
    }
    self.merge_opacity(r, opacity) if spot and spot != self
    r
  end
  
  def self.public_list_where list
    'panels.publish > 0'
  end
  
  def self.show_opt
    {:include => {:panel => {:author => {}} }}
  end
  
  def scenario
    if caption.blank?
      ''
    else
      '<p>' + ERB::Util.html_escape(self.caption) + '</p>'
    end
  end
  
  def plain_scenario
    if caption.blank?
      ''
    else
      self.caption + "\n"
    end
  end
  
end
